@@ -9,13 +9,15 @@ use std::rc::Rc;
 use anyhow::{anyhow, Context};
 use cargo::core::{Dependency, PackageId, Summary, Verbosity, Workspace};
 use cargo::ops::{update_lockfile, UpdateOptions};
+use cargo::sources::config::SourceConfigMap;
 use cargo::util::{CargoResult, Config};
-use semver::{Identifier, Version, VersionReq};
+use semver::{Version, VersionReq};
 use tempfile::{Builder, TempDir};
 use toml::value::Table;
 use toml::Value;
 
 use super::{ElaborateWorkspace, Manifest};
+use crate::error::OutdatedError;
 use crate::Options;
 
 /// A temporary project
@@ -191,7 +193,7 @@ impl<'tmp> TempProject<'tmp> {
             options.flag_color.as_deref(),
             options.frozen(),
             options.locked(),
-            false,
+            options.flag_offline,
             &cargo_home_path,
             &[],
             &[],
@@ -209,7 +211,13 @@ impl<'tmp> TempProject<'tmp> {
             dry_run: false,
             workspace: self.is_workspace_project,
         };
-        update_lockfile(self.workspace.borrow().as_ref().unwrap(), &update_opts)?;
+        update_lockfile(
+            self.workspace
+                .borrow()
+                .as_ref()
+                .ok_or(OutdatedError::NoWorkspace)?,
+            &update_opts,
+        )?;
         Ok(())
     }
 
@@ -280,7 +288,7 @@ impl<'tmp> TempProject<'tmp> {
             }
             Self::manipulate_dependencies(&mut manifest, &|deps| {
                 Self::replace_path_with_absolute(
-                    &self,
+                    self,
                     deps,
                     orig_root.as_ref(),
                     tmp_root.as_ref(),
@@ -331,7 +339,7 @@ impl<'tmp> TempProject<'tmp> {
             }
             Self::manipulate_dependencies(&mut manifest, &|deps| {
                 Self::replace_path_with_absolute(
-                    &self,
+                    self,
                     deps,
                     orig_root.as_ref(),
                     tmp_root.as_ref(),
@@ -363,12 +371,13 @@ impl<'tmp> TempProject<'tmp> {
         let package_id = workspace.find_direct_dependency(name, dependent_package_name)?;
         let version = package_id.version();
         let source_id = package_id.source_id().with_precise(None);
-        let mut source = source_id.load(&self.config, &HashSet::new())?;
+        let source_config = SourceConfigMap::new(workspace.workspace.config())?;
+        let mut source = source_config.load(source_id, &HashSet::new())?;
         if !source_id.is_default_registry() {
             let _lock = self.config.acquire_package_cache_lock()?;
             source.update()?;
         }
-        let dependency = Dependency::parse_no_deprecated(name, None, source_id)?;
+        let dependency = Dependency::parse(name, None, source_id)?;
         let query_result = {
             let _lock = self.config.acquire_package_cache_lock()?;
             let mut query_result = source.query_vec(&dependency)?;
@@ -379,21 +388,25 @@ impl<'tmp> TempProject<'tmp> {
             Some(requirement) => Some(VersionReq::parse(requirement)?),
             None => None,
         };
-        let latest_result = query_result
-            .iter()
-            .find(|summary| {
-                if summary.version() < version {
-                    false
-                } else if version_req.is_none() {
-                    true
-                } else if find_latest {
-                    self.options.flag_aggressive
-                        || valid_latest_version(requirement.unwrap(), summary.version())
-                } else {
-                    version_req.as_ref().unwrap().matches(summary.version())
-                }
-            })
-            .unwrap_or_else(|| {
+        let latest_result = query_result.iter().find(|summary| {
+            if summary.version() < version {
+                false
+            } else if version_req.is_none() {
+                true
+            } else if find_latest {
+                // this unwrap is safe since we check if `version_req` is `None` before this
+                // (which is only `None` if `requirement` is `None`)
+                self.options.flag_aggressive
+                    || valid_latest_version(requirement.unwrap(), summary.version())
+            } else {
+                // this unwrap is safe since we check if `version_req` is `None` before this
+                version_req.as_ref().unwrap().matches(summary.version())
+            }
+        });
+
+        let latest_summary = match latest_result {
+            Some(summary) => summary,
+            None => {
                 // If the version_req cannot be found use the version
                 // this happens when we use a git repository as a dependency, without specifying
                 // the version in Cargo.toml, preventing us from needing an unwrap below in the warn
@@ -401,18 +414,22 @@ impl<'tmp> TempProject<'tmp> {
                     Some(v_r) => format!("{}", v_r),
                     None => format!("{}", version),
                 };
-
+                // this should be safe it should only fail if we cannot get
+                // access to write to the terminal
+                // if this fails it's a cargo (as a dependency) issue
                 self.warn(format!(
                     "cannot compare {} crate version found in toml {} with crates.io latest {}",
                     name,
                     ver_req,
                     query_result[0].version()
-                ))
-                .unwrap();
+                ))?;
+
                 //this returns the latest version
                 &query_result[0]
-            });
-        Ok(latest_result.clone())
+            }
+        };
+
+        Ok(latest_summary.clone())
     }
 
     fn feature_includes(&self, name: &str, optional: bool, features_table: &Option<Value>) -> bool {
@@ -482,7 +499,10 @@ impl<'tmp> TempProject<'tmp> {
                 continue;
             }
 
-            let original = dependencies.get(&dep_key).cloned().unwrap();
+            let original = dependencies
+                .get(&dep_key)
+                .cloned()
+                .ok_or(OutdatedError::NoMatchingDependency)?;
 
             match original {
                 Value::String(requirement) => {
@@ -533,7 +553,7 @@ impl<'tmp> TempProject<'tmp> {
                             }
                         })
                         .unwrap_or(false);
-                    if !self.feature_includes(&name, optional, features) {
+                    if !self.feature_includes(name, optional, features) {
                         continue;
                     }
                     let mut replaced = t.clone();
@@ -544,9 +564,9 @@ impl<'tmp> TempProject<'tmp> {
                     };
                     let r_summary = self.find_update(
                         if orig_name.is_empty() {
-                            &name
+                            name
                         } else {
-                            &orig_name
+                            orig_name
                         },
                         package_name,
                         requirement,
@@ -580,6 +600,9 @@ impl<'tmp> TempProject<'tmp> {
                                     };
                                     let retained =
                                         features_and_options(&summary).contains(feature.as_str());
+                                    // this unwrap should be safe it should only fail if we cannot get
+                                    // access to write to the terminal
+                                    // if this fails it's a cargo (as a dependency) issue
                                     if !retained {
                                         self.warn(format!(
                                             "Feature {} of package {} \
@@ -618,7 +641,10 @@ impl<'tmp> TempProject<'tmp> {
     ) -> CargoResult<()> {
         let dep_names: Vec<_> = dependencies.keys().cloned().collect();
         for name in dep_names {
-            let original = dependencies.get(&name).cloned().unwrap();
+            let original = dependencies
+                .get(&name)
+                .cloned()
+                .ok_or(OutdatedError::NoMatchingDependency)?;
             match original {
                 Value::Table(ref t) if t.contains_key("path") => {
                     if let Value::String(ref orig_path) = t["path"] {
@@ -746,7 +772,7 @@ fn manifest_paths(elab: &ElaborateWorkspace<'_>) -> CargoResult<Vec<PathBuf>> {
 }
 
 fn valid_latest_version(mut requirement: &str, version: &Version) -> bool {
-    match (requirement.contains('-'), version.is_prerelease()) {
+    match (requirement.contains('-'), !version.pre.is_empty()) {
         // if user was on a stable channel, it's unlikely for him to update to an unstable one
         (false, true) => false,
         // both are stable, leave for further filters
@@ -756,14 +782,18 @@ fn valid_latest_version(mut requirement: &str, version: &Version) -> bool {
         // both are unstable, must be in the same channel
         (true, true) => {
             requirement = requirement.trim_start_matches(&['=', ' ', '~', '^'][..]);
-            let requirement_version = Version::parse(&requirement)
+            let requirement_version = Version::parse(requirement)
                 .expect("Error could not parse requirement into a semantic version");
-            let requirement_channel = requirement_version.pre[0].to_string();
-            match (requirement_channel.is_empty(), &version.pre[0]) {
-                (true, &Identifier::Numeric(_)) => true,
-                (false, &Identifier::AlphaNumeric(_)) => {
-                    Identifier::AlphaNumeric(requirement_channel) == version.pre[0]
-                }
+            let requirement_channel = requirement_version.pre.split('.').next().unwrap();
+            let requirement_channel_numeric =
+                requirement_channel.bytes().all(|b| b.is_ascii_digit());
+
+            let version_channel = version.pre.split('.').next().unwrap();
+            let version_channel_numeric = version_channel.bytes().all(|b| b.is_ascii_digit());
+
+            match (requirement_channel_numeric, version_channel_numeric) {
+                (false, false) => requirement_channel == version_channel,
+                (true, true) => true,
                 _ => false,
             }
         }
